""" All constants needed for signal operations, can be called outside of this package
"""
from mongoengine import signals


pre_init = signals.pre_init
post_init = signals.post_init
pre_save = signals.pre_save
pre_save_post_validation = signals.pre_save_post_validation
post_save = signals.post_save
pre_delete = signals.pre_delete
post_delete = signals.post_delete
pre_bulk_insert = signals.pre_bulk_insert
post_bulk_insert = signals.post_bulk_insert



""" Connect a signal/signals to a receiver.
"""
from signals_utils.commons.exceptions import SignalsError


def connect(receiver, signal, sender):
    """ Connecting receiver to signals. Used by passing in the receiver (method/function), signal (or list of signals)
    and the sender (class to watch).

    Example::

        connector.connect(receiver, signals.post_save, MyModel)
        connector.connect(receiver, [signals.post_save, signals.post_delete], MyModel)

        def receiver(sender, document, **kwargs):
            ...

    Args:
        receiver: Method/Function to receive the signal/signals.
        signal: Signal or list of signals.
        sender: Class to watch.

    Raises:
        SignalsError: Error(s) during the connection.

    """
    try:
        if isinstance(signal, (list, tuple)):
            for s in signal:
                s.connect(receiver=receiver, sender=sender)
        else:
            signal.connect(receiver=receiver, sender=sender)
    except Exception as e:
        raise SignalsError(e)



""" Signals Utils Exceptions
"""


class SignalsError(Exception):
    """ Exception raised when a signal connection (binding) fails.
    """
    def __init__(self, message):
        self.message = message

    def __str__(self):
        return repr(self.message)
